@@ -1,6 +1,7 @@
-#![doc(html_root_url = "http://docs.rs/const-default/0.2.0")]
+#![doc(html_root_url = "http://docs.rs/const-default/0.3.0")]
 #![cfg_attr(not(feature = "std"), no_std)]
 #![cfg_attr(all(feature = "unstable", feature = "alloc"), feature(const_btree_new))]
+#![cfg_attr(feature = "unstable", feature(cfg_target_has_atomic))]
 #![cfg_attr(feature = "unstable", allow(incomplete_features))]
 
 #[cfg(feature = "alloc")]
@@ -9,7 +10,30 @@ extern crate alloc;
 #[cfg(feature = "derive")]
 pub use const_default_derive::ConstDefault;
 
+/// Implements a compilation time default value for the implemented type.
+///
+/// # Note
+///
+/// Unlike the [`Default`] trait implementation the `DEFAULT` of implementations
+/// of this trait can be used in constant evaluation contexts.
+///
+/// # Example
+///
+/// ```
+/// # #[cfg(feature = "std")]
+/// # const _: () = {
+/// # use const_default::ConstDefault;
+/// const VEC: Vec<u8> = <Vec<u8> as ConstDefault>::DEFAULT;
+/// # };
+/// ```
+///
+/// The above code works while the below code does not:
+///
+/// ```compile_fail
+/// const VEC: Vec<u8> = <Vec<u8> as Default>::default();
+/// ```
 pub trait ConstDefault: Sized {
+	/// The constant default value.
 	const DEFAULT: Self;
 }
 
@@ -24,8 +48,13 @@ impl<T> ConstDefault for Option<T> {
 }
 
 #[cfg(feature = "alloc")]
-impl<'a, T: ConstDefault + Clone + 'a> ConstDefault for alloc::borrow::Cow<'a, T> {
-	const DEFAULT: Self = alloc::borrow::Cow::Owned(T::DEFAULT);
+impl<'a, T> ConstDefault for Cow<'a, T>
+where
+	T: ToOwned + ?Sized + 'a,
+	<T as ToOwned>::Owned: ConstDefault,
+{
+	const DEFAULT: Self =
+		Self::Owned(<<T as ToOwned>::Owned as ConstDefault>::DEFAULT);
 }
 
 impl<T: ConstDefault> ConstDefault for core::cell::Cell<T> {
@@ -110,6 +139,10 @@ impl<T: ?Sized> ConstDefault for core::marker::PhantomData<T> {
 	const DEFAULT: Self = Self;
 }
 
+impl ConstDefault for core::marker::PhantomPinned {
+	const DEFAULT: Self = Self;
+}
+
 impl<T> ConstDefault for core::iter::Empty<T> {
 	const DEFAULT: Self = core::iter::empty();
 }
@@ -128,7 +161,7 @@ impl ConstDefault for std::sync::Once {
 }
 
 macro_rules! impl_num {
-	($($ty:ty=$d:expr$(;$name:ident)?),*) => {
+	($($ty:ty=$d:expr$(;$name:ident=$width:literal)?),*) => {
 		$(
 			impl ConstDefault for $ty {
 				const DEFAULT: Self = $d;
@@ -139,8 +172,9 @@ macro_rules! impl_num {
 			}
 
 			$(
-				#[cfg(feature = "std")]
-				impl ConstDefault for std::sync::atomic::$name {
+				#[cfg(feature = "enable-atomics")]
+				#[cfg_attr(feature = "unstable", cfg(target_has_atomic = $width))]
+				impl ConstDefault for core::sync::atomic::$name {
 					const DEFAULT: Self = Self::new(ConstDefault::DEFAULT);
 				}
 			)?
@@ -150,16 +184,23 @@ macro_rules! impl_num {
 
 impl_num! {
 	()=(), bool=false, f32=0.0, f64=0.0, char='\x00', &str="",
-	u8=0;AtomicU8, u16=0;AtomicU16, u32=0;AtomicU32, u64=0;AtomicU64, usize=0;AtomicUsize,
-	i8=0;AtomicI8, i16=0;AtomicI16, i32=0;AtomicI32, i64=0;AtomicI64, isize=0;AtomicIsize,
+	u8=0;AtomicU8="8", u16=0;AtomicU16="16", u32=0;AtomicU32="32", u64=0;AtomicU64="64", usize=0;AtomicUsize="ptr",
+	i8=0;AtomicI8="8", i16=0;AtomicI16="16", i32=0;AtomicI32="32", i64=0;AtomicI64="64", isize=0;AtomicIsize="ptr",
 	i128=0, u128=0
 }
 
-#[cfg(feature = "std")]
-impl ConstDefault for std::sync::atomic::AtomicBool {
+#[cfg(feature = "enable-atomics")]
+#[cfg_attr(feature = "unstable", cfg(target_has_atomic = "8"))]
+impl ConstDefault for core::sync::atomic::AtomicBool {
 	const DEFAULT: Self = Self::new(ConstDefault::DEFAULT);
 }
 
+#[cfg(feature = "enable-atomics")]
+#[cfg_attr(feature = "unstable", cfg(target_has_atomic = "ptr"))]
+impl<T> ConstDefault for core::sync::atomic::AtomicPtr<T> {
+	const DEFAULT: Self = Self::new(core::ptr::null_mut());
+}
+
 macro_rules! impl_tuple {
 	(@rec $t:ident) => { };
 	(@rec $_:ident $($t:ident)+) => {
